
           LIST P=16F873           ;  16F873 Runs at 4.000 MHz
           INCLUDE "P16f873.inc"
          

    

;  Define Information
     #DEFINE RS PORTC, 2
     #DEFINE E  PORTC, 3
     #DEFINE Key1 PORTC, 5
     #DEFINE Key2 PORTC, 6
     #DEFINE Key3 PORTC, 7
;  Macro

EStrobe MACRO                   ;  Strobe the "E" Bit
  bsf    E
  bcf    E
 ENDM

            CBLOCK     0x20
                Dlay            ; 8 Bit Delay Variable
                Temp            ; a temporary variable
                cntmsec
                LED1
                LED2
                LED3
                LED4
                LED5
                ELED1
                ELED2
                ELED3
                ELED4
                ELED5
                NumH
                NumL
                Tens
                Ones
                Thou
                TenK
                Hund
                FrHig
                FrLow
                Temp1
                Temp2
                Temp3
                Temp4
                SelFr
                Marker
                Select
                AdresEEP
                BufOutEEP
                BufInEEP
                SelNum
                ELEDKM
                Time1
                TempR
                LEDS
               ENDC
               bsf  PCLATH,3
               call CLV            ;Clear all variadles
           
               ORG 4               ; start at location 0
           
            goto main           ; jump over to main routine       

            
ClearVar             
           clrf LED1
           clrf LED2
           clrf LED3
           clrf LED4
           clrf LED5
           clrf ELED1
           clrf ELED2
           clrf ELED3
           clrf ELED4
           clrf ELED5
           clrf NumH
           clrf NumL
           clrf Tens
           clrf Ones
           clrf Thou
           clrf TenK
           clrf Hund 
           clrf SelFr
                    
           return

;----------------------------------------------------------------------;
;            Initialize the ports and timer1                           ;
;----------------------------------------------------------------------;
init:
            
            bsf    STATUS,RP0
            movlw  0xFF                ;  PortA all inputs
            movwf  TRISA
            movlw  0x00                ;  all outputs on port B
            movwf  TRISB
            movlw B'11110011'          ;RC2,RC3 outputs other inputs
            movwf  TRISC
            bcf    STATUS,RP0
            clrf   PORTA
            clrf   PORTB
            clrf   PORTC
            movlw  B'00000110'
            movwf  T1CON
            bcf    T1CON,0
            
          
            return

;----------------------------------------------------------------------;
;                 Initialize the LCD                                   ;
;----------------------------------------------------------------------;
initlcd:
  movlw D'40'
  call   nmsec                  ;  Wait 40 msecs before Reset
  bcf    RS                     ;  send an 8 bit instruction
  movlw  0x03                   ;  Reset Command
  call   NybbleOut              ;  Send the Nybble
  call   Dlay5                  ;  Wait 5 msecs before Sending Again
  EStrobe
  call Dlay160                  ;  Wait 160 usecs before Sending 2nd Time
  EStrobe
  call Dlay160                  ;  Wait 160 usecs before Sending 3rd Time
  bcf    RS                     ;  send an 8 bit instruction
  movlw  0x02                   ;  Set 4 Bit Mode
  call   NybbleOut              
  call Dlay160
  movlw  0x028                  ;  4 bit, 2 Line, 5x7 font
  call   SendINS
  movlw  0x010                  ;  display shift off
  call   SendINS
  movlw  0x001                  ;  Clear the Display RAM
  call   SendINS
  call   Dlay5                  ;  Note, Can take up to 4.1 msecs
 
  return

;----------------------------------------------------------------------;
;              Send the character in W out to the LCD                  ;
;----------------------------------------------------------------------;
SendASCII
                                ;  Send nbr as ASCII character
                                ;  Send the Character to the LCD
  movwf  Temp                   ;  Save the Temporary Value
  swapf  Temp, w                ;  Send the High Nybble
  bsf    RS                     ;  RS = 1
  call   NybbleOut
  movf   Temp, w                ;  Send the Low Nybble
  bsf    RS
  call   NybbleOut
  return

;----------------------------------------------------------------------;
;              Send an instruction in W out to the LCD                 ;
;----------------------------------------------------------------------;
SendINS                         ;  Send the Instruction to the LCD
  movwf  Temp                   ;  Save the Temporary Value
  swapf  Temp, w                ;  Send the High Nybble
  bcf    RS                     ;  RS = 0
  call   NybbleOut
  movf   Temp, w                ;  Send the Low Nybble
  bcf    RS
  call   NybbleOut
  return

;----------------------------------------------------------------------;
;              Send the nibble in W out to the LCD                     ;
;----------------------------------------------------------------------;
NybbleOut                       ;  Send a Nybble to the LCD
  andlw  B'00001111'
  movwf  PORTB                            
  EStrobe                       ;  Strobe out the LCD Data
  call Dlay160                  ;  delay for 160 usec
  return


;----------------------------------------------------------------------;
;                        time delay routines                           ;
;----------------------------------------------------------------------;

Dlay160:    movlw D'41'               
micro4      addlw H'FF'                
            btfss STATUS,Z             
            goto micro4                
            return                     

Dlay5:      movlw 5                    
            goto $ + 2
msec250:    movlw D'250'               
            
nmsec:      movwf cntmsec              
msecloop:   movlw D'254'              
            bsf   PCLATH,0
            bsf   PCLATH,3
            call  ScanKeyR
            call micro4              
                                       
                                       
                                      
            decfsz cntmsec, f          
            goto msecloop              
SER            
            return                     
                                       
;----------------------------------------------------------------------;
;                        time delay 200uS                              ;
;----------------------------------------------------------------------;
Delay200
         movlw D'100'
         movwf Temp2
         decfsz Temp2,1
         goto $-1
         return
;----------------------------------------------------------------------;
;                              LCD  Line1                              ;
;----------------------------------------------------------------------;
Line1
       movlw  0x080                 
       call   SendINS
       movlw  0x00C                  
       call   SendINS
       return
;----------------------------------------------------------------------;
;                              LCD  Line1                              ;
;----------------------------------------------------------------------;
Line2
       movlw  0x0C0                 
       call   SendINS
       movlw  0x00C                  
       call   SendINS
       return

;-----------------------------------------------------------------------;
;                Out to LCD frequence xxx.xx kHz                       ;
;-----------------------------------------------------------------------; 
FreqkHz
       
       movf  LED1,0
       addlw "0"
       call  SendASCII 
       movf  LED2,0
       addlw "0"
       call  SendASCII 
       movf  LED3,0
       addlw "0"
       call  SendASCII 
       movlw "."
       call  SendASCII
       movf  LED4,0
       addlw "0"
       call  SendASCII 
       movf  LED5,0
       addlw "0"
       call  SendASCII 
       movlw " "
       call  SendASCII
       movlw .6
       xorwf SelNum,0
       btfss STATUS,Z
       goto  Frb2
       btfsc Time1,3
       goto  Frb1
Frb2
       movlw "k"
       call  SendASCII
       movlw "H"
       call  SendASCII
       movlw "z"
       call  SendASCII
       return
Frb1    
       
       movlw " "
       call  SendASCII
       movlw "H"
       call  SendASCII
       movlw "z"
       call  SendASCII
       return
;-----------------------------------------------------------------------;
;                Out to LCD frequence x.xxxx MHz                       ;
;-----------------------------------------------------------------------; 
FreqMHz1
       
       movf  LED1,0
       addlw "0"
       call  SendASCII
       movlw "."
       call  SendASCII 
       movf  LED2,0
       addlw "0"
       call  SendASCII 
       movf  LED3,0
       addlw "0"
       call  SendASCII 
       movf  LED4,0
       addlw "0"
       call  SendASCII 
       movf  LED5,0
       addlw "0"
       call  SendASCII 
       movlw " "
       call  SendASCII
       movlw .6
       xorwf SelNum,0
       btfss STATUS,Z
       goto  Frb4
       btfsc Time1,3
       goto  Frb3
Frb4
       movlw "M"
       call  SendASCII
       movlw "H"
       call  SendASCII
       movlw "z"
       call  SendASCII
       return
Frb3    
       
       movlw " "
       call  SendASCII
       movlw "H"
       call  SendASCII
       movlw "z"
       call  SendASCII
       return
;-----------------------------------------------------------------------;
;                Out to LCD frequence xx.xxx MHz                       ;
;-----------------------------------------------------------------------; 
FreqMHz2
       
       movf  LED1,0
       addlw "0"
       call  SendASCII
       movf  LED2,0
       addlw "0"
       call  SendASCII
       movlw "."
       call  SendASCII 
       movf  LED3,0
       addlw "0"
       call  SendASCII 
       movf  LED4,0
       addlw "0"
       call  SendASCII 
       movf  LED5,0
       addlw "0"
       call  SendASCII 
       movlw " "
       call  SendASCII
       movlw .6
       xorwf SelNum,0
       btfss STATUS,Z
       goto  Frb6
       btfsc Time1,3
       goto  Frb5
Frb6
       movlw "M"
       call  SendASCII
       movlw "H"
       call  SendASCII
       movlw "z"
       call  SendASCII
       return
Frb5    
       
       movlw " "
       call  SendASCII
       movlw "H"
       call  SendASCII
       movlw "z"
       call  SendASCII
       return

;----------------------------------------------------------------------;
;                            Text "Fin: "                              ;
;----------------------------------------------------------------------;       
Text1
       movlw  "F"
       call  SendASCII
       movlw  "i"
       call  SendASCII
       movlw  "n"
       call  SendASCII
       movlw  ":"
       call  SendASCII
       movlw  " "
       call  SendASCII
       return
;----------------------------------------------------------------------;
;                            Text "F+M0 "                              ;
;----------------------------------------------------------------------;       
Text3
       movlw  "F"
       call  SendASCII
       btfss  LEDS,0
       movlw  "+"
       btfsc  LEDS,0
       movlw  "-"
       call  SendASCII
       movlw  "M"
       call  SendASCII
       movf  Marker,0
       addlw  "0"
       call  SendASCII
       movlw  " "
       call  SendASCII
       return
;----------------------------------------------------------------------;
;                            Text "M0 : "                              ;
;----------------------------------------------------------------------;       
Text5
       movlw  "M"
       call  SendASCII
       movf  Marker,0
       addlw  "0"
       call  SendASCII
       movlw  " "
       call  SendASCII
       movlw  ":"
       call  SendASCII       
       movlw  " "
       call  SendASCII
       return
;----------------------------------------------------------------------;
;                     Text "Set    M+    M- "                          ;
;----------------------------------------------------------------------;       
Text2
       movlw  "S"
       call  SendASCII
       movlw  "e"
       call  SendASCII
       movlw  "t"
       call  SendASCII
       movlw  " "
       call  SendASCII
       movlw  " "
       call  SendASCII
       movlw  " "
       call  SendASCII
       movlw  " "
       call  SendASCII
       movlw  "M"
       call  SendASCII
       movlw  "+"
       call  SendASCII
       movlw  " "
       call  SendASCII
       movlw  " "
       call  SendASCII
       movlw  " "
       call  SendASCII
       movlw  " "
       call  SendASCII
       movlw  "M"
       call  SendASCII
       movlw  "-"
       call  SendASCII
       
       return
;----------------------------------------------------------------------;
;                     Text "      Yes   No "                          ;
;----------------------------------------------------------------------;       
Text7
       movlw  " "
       call  SendASCII
       movlw  " "
       call  SendASCII
       movlw  " "
       call  SendASCII
       movlw  " "
       call  SendASCII
       movlw  " "
       call  SendASCII
       movlw  " "
       call  SendASCII
       movlw  "Y"
       call  SendASCII
       movlw  "e"
       call  SendASCII
       movlw  "s"
       call  SendASCII
       movlw  " "
       call  SendASCII
       movlw  " "
       call  SendASCII
       movlw  " "
       call  SendASCII
       movlw  " "
       call  SendASCII
       movlw  "N"
       call  SendASCII
       movlw  "o"
       call  SendASCII
       
       return
;----------------------------------------------------------------------;
;                     Text "Save marker 0 ?"                          ;
;----------------------------------------------------------------------;       
Text6
       movlw  "S"
       call  SendASCII
       movlw  "a"
       call  SendASCII
       movlw  "v"
       call  SendASCII
       movlw  "e"
       call  SendASCII
       movlw  " "
       call  SendASCII
       movlw  "m"
       call  SendASCII
       movlw  "a"
       call  SendASCII
       movlw  "r"
       call  SendASCII
       movlw  "k"
       call  SendASCII
       movlw  "e"
       call  SendASCII
       movlw  "r"
       call  SendASCII
       movlw  " "
       call  SendASCII
       movf  Marker,0
       addlw  "0"
       call  SendASCII
       movlw  " "
       call  SendASCII
       movlw  "?"
       call  SendASCII
       movlw  " "
       call  SendASCII
       
       return
;----------------------------------------------------------------------;
;                     Text "Set     ^     > "                          ;
;----------------------------------------------------------------------;       
Text4
       movlw  "S"
       call  SendASCII
       movlw  "e"
       call  SendASCII
       movlw  "t"
       call  SendASCII
       movlw  " "
       call  SendASCII
       movlw  " "
       call  SendASCII
       movlw  " "
       call  SendASCII
       movlw  " "
       call  SendASCII
       movlw  " "
       call  SendASCII
       movlw  "^"
       call  SendASCII
       movlw  " "
       call  SendASCII
       movlw  " "
       call  SendASCII
       movlw  " "
       call  SendASCII
       movlw  " "
       call  SendASCII
       movlw  " "
       call  SendASCII
       movlw  ">"
       call  SendASCII
       
       return
;----------------------------------------------------------------------;
;                Convert two bute to dec                               ;
;----------------------------------------------------------------------;
Convert			
	movf    NumH,0		
	swapf	NumH,w
	andlw	0x0F 	
	addlw	0xF0 	
	movwf	Thou
	addwf	Thou,f
	addlw	0xE2
	movwf	Hund
	addlw	0x32
	movwf	Ones

	movf	NumH,w
	andlw	0x0F
	addwf	Hund,f
	addwf	Hund,f
	addwf	Ones,f
	addlw	0xE9
	movwf	Tens
	addwf	Tens,f
	addwf	Tens,f

	swapf	NumL,w
	andlw	0x0F
	addwf	Tens,f
	addwf	Ones,f

	rlf	Tens,f
	rlf	Ones,f
	comf	Ones,f
	rlf	Ones,f

	movf	NumL,w
	andlw	0x0F
	addwf	Ones,f
	rlf	Thou,f

	movlw	0x07
	movwf	TenK


	movlw	0A	 
Lb1:
	addwf	Ones,f
	decf	Tens,f
	btfss	3,0
	goto	Lb1
Lb2:
	addwf	Tens,f
	decf	Hund,f
	btfss	3,0
	goto	Lb2
Lb3:
	addwf	Hund,f
	decf	Thou,f
	btfss	3,0
	goto	Lb3
Lb4:
	addwf	Thou,f
	decf	TenK,f
	btfss	3,0
	goto	Lb4
	retlw   00
;----------------------------------------------------------------------;
;                           Scan keys                                  ;
;----------------------------------------------------------------------;
ScanKey
        btfsc  Key2
        incf   Marker,1
        btfsc  Key3
        decf   Marker,1
ScanK1
        btfsc  Key1
        incf   Select,1
        btfsc  Key2
        call   Zummer
        btfsc  Key3
        call   Zummer
        btfsc  Key1
        call   Zummer
        return
;----------------------------------------------------------------------;
;                           Scan keys (press)                          ;
;----------------------------------------------------------------------;
ScanKP
        btfsc  Key2
        goto   $-1
        btfsc  Key3
        goto   $-1
        btfsc  Key1
        goto   $-1
        
        return
;----------------------------------------------------------------------;
;                Get frequence to LED1...LED5                          ;
;----------------------------------------------------------------------;
OutFrDis
       
        movf    FrHig,0
        movwf   NumH
        movf    FrLow,0
        movwf   NumL
      
        call    Convert
        movf    TenK,0
        movwf   LED1
        movf    Thou,0
        movwf   LED2
        movf    Hund,0
        movwf   LED3
        movf    Tens,0
        movwf   LED4
        movf    Ones,0
        movwf   LED5
        return
;----------------------------------------------------------------------;
;                  Get markers to symbol -/+                           ;
;----------------------------------------------------------------------;
SymbMark
        btfsc LEDS,0
        movlw "-"
        btfss LEDS,0
        movlw "+"
        btfsc LEDS,7
        movlw " "
        call  SendASCII
        
        
        return
;----------------------------------------------------------------------;
;                  Get markers to LED1...LED5                          ;
;----------------------------------------------------------------------;
OutMarkDis
        
        movlw   .0
        xorwf  SelNum,0
        btfsc  STATUS,Z
        goto   AddLS
        movlw   .1
        xorwf  SelNum,0
        btfsc  STATUS,Z
        goto   AddL1
        movlw   .2
        xorwf  SelNum,0
        btfsc  STATUS,Z
        goto   AddL2
        movlw   .3
        xorwf  SelNum,0
        btfsc  STATUS,Z
        goto   AddL3
        movlw   .4
        xorwf  SelNum,0
        btfsc  STATUS,Z
        goto   AddL4
        movlw   .5
        xorwf  SelNum,0
        btfsc  STATUS,Z
        goto   AddL5
        movlw   .6
        xorwf  SelNum,0
        btfsc  STATUS,Z
        goto   AddLKM
        goto   EndL
AddLS
        btfsc Key2
        incf LEDS,1
        goto  EndL
AddL1
        btfsc  Key2
        incf   ELED1,1
        movf   ELED1,0
        goto   EndL1
AddL2
        btfsc  Key2
        incf   ELED2,1
        movf   ELED2,0
        goto   EndL2
AddL3
        btfsc  Key2
        incf   ELED3,1
        movf   ELED3,0
        goto   EndL3
AddL4
        btfsc  Key2
        incf   ELED4,1
        movf   ELED4,0
        goto   EndL4
AddL5
        btfsc  Key2
        incf   ELED5,1
        movf   ELED5,0
        goto   EndL5
AddLKM
        btfsc  Key2
        incf   ELEDKM,1
        movf   ELEDKM,0
        goto   EndLKM
EndL1    
        sublw  .9
        btfss  STATUS,C
        clrf   ELED1
        goto   EndL
EndL2    
        sublw  .9
        btfss  STATUS,C
        clrf   ELED2
        goto   EndL
EndL3    
        sublw  .9
        btfss  STATUS,C
        clrf   ELED3
        goto   EndL
EndL4    
        sublw  .9
        btfss  STATUS,C
        clrf   ELED4
        goto   EndL
EndL5    
        sublw  .9
        btfss  STATUS,C
        clrf   ELED5
        goto   EndL
EndLKM    
        sublw  .2
        btfss  STATUS,C
        clrf   ELEDKM
        goto   EndL
EndL
        movf    ELED1,0
        movwf   LED1
        movf    ELED2,0
        movwf   LED2
        movf    ELED3,0
        movwf   LED3
        movf    ELED4,0
        movwf   LED4
        movf    ELED5,0
        movwf   LED5
       
        return
;----------------------------------------------------------------------;
;                Timer1 count pulse in RC0  kHz                        ;
;----------------------------------------------------------------------;
Count_kHz       
       
       clrf    FrLow
       clrf    FrHig
       movlw   .174
       movwf   Temp1
       clrf    TMR1H
       clrf    TMR1L
       bcf     PIR1,0
       bsf     T1CON,0
DelayT10
       movlw   .190
       movwf   Temp2
       decfsz  Temp2,1
       goto    $-1
       decfsz  Temp1,1
       goto    DelayT10
       movlw   .41
       movwf   Temp3
       decfsz  Temp3,1
       goto    $-1
       bcf     T1CON,0
       movf    TMR1H,0
       movwf   FrHig
       movf    TMR1L,0
       movwf   FrLow
       clrf    SelFr
       return
;----------------------------------------------------------------------;
;                Timer1 count pulse in RC0  xMHz                       ;
;----------------------------------------------------------------------;
Count_MHz1       
       
       clrf    FrLow
       clrf    FrHig
       movlw   .86
       movwf   Temp1
       clrf    TMR1H
       clrf    TMR1L
       bcf     PIR1,0
       bsf     T1CON,0
DelayT1
       movlw   .36
       movwf   Temp2
       decfsz  Temp2,1
       goto    $-1
       decfsz  Temp1,1
       goto    DelayT1
       movlw   .122
       movwf   Temp3
       decfsz  Temp3,1
       goto    $-1
       nop
       bcf     T1CON,0
       movf    TMR1H,0
       movwf   FrHig
       movf    TMR1L,0
       movwf   FrLow
       clrf    SelFr
       bsf     SelFr,0
       return
;----------------------------------------------------------------------;
;                Timer1 count pulse in RC0  xMHz                       ;
;----------------------------------------------------------------------;
Count_MHz2       
       movlw   B'00010110'
       movwf   T1CON
       clrf    FrLow
       clrf    FrHig
       bcf     PIR1,0
       clrf    TMR1H
       clrf    TMR1L
       bsf     T1CON,0
       movlw   .250
       movwf   Temp3
       decfsz  Temp3,1
       goto    $-1
       movlw   .250
       movwf   Temp3
       decfsz  Temp3,1
       goto    $-1
       movlw   .164
       movwf   Temp3
       decfsz  Temp3,1
       goto    $-1
       nop
       nop
       nop
       nop
       bcf     T1CON,0
       movf    TMR1H,0
       movwf   FrHig
       movf    TMR1L,0
       movwf   FrLow
       movlw   B'00000110'
       movwf   T1CON
       clrf    SelFr
       bsf     SelFr,1
       return
;----------------------------------------------------------------------;
;                           Zummer sound                               ;
;----------------------------------------------------------------------;
Zummer 
       movlw .50
       movwf Temp1
zret   bcf   PORTB,5  
       bsf   PORTB,4
       call   Delay200
       bsf   PORTB,5
       bcf   PORTB,4
       call   Delay200
       decfsz Temp1,1
       goto   zret
       return
;----------------------------------------------------------------------;
;                     EEPROM data read                                 ;
;----------------------------------------------------------------------;
ReadEEPROM
      bsf  STATUS,RP1
      bcf  STATUS,RP0
      movf AdresEEP,W
      movwf EEADR 
      bsf  STATUS,RP0
      bcf  EECON1,EEPGD
      bsf  EECON1,RD
      bcf  STATUS,RP0 
      movf EEDATA,W
      movwf BufOutEEP
      bcf  STATUS,RP1
      return
;----------------------------------------------------------------------;
;                     EEPROM data write                                ;
;----------------------------------------------------------------------;
WriteEEPROM
     bsf  STATUS,RP1
     bsf  STATUS,RP0
     btfsc EECON1,WR
     goto  $-1
     bcf  STATUS,RP0
     movf AdresEEP,W
     movwf EEADR
     movf BufInEEP,0
     movwf EEDATA
     bsf  STATUS,RP0
     bcf  EECON1,EEPGD
     bsf  EECON1,WREN
     bcf  INTCON,GIE
     movlw 0x55
     movwf EECON2
     movlw 0xAA
     movwf EECON2
     bsf   EECON1,WR
     bsf   INTCON,GIE
     bcf   EECON1,WREN
     bcf   STATUS,RP1
     bcf   STATUS,RP0
     return
;----------------------------------------------------------------------;
;                   Read byte ELES..ELED5                              ;
;----------------------------------------------------------------------;
ReadByte
     clrf  Temp1
     movf  Marker,0
     movwf Temp2
LoopR
     movlw  .7
     addwf  Temp1,1
     decfsz Temp2,1
     goto   LoopR
     movf  Temp1,0
     movwf AdresEEP
     call  ReadEEPROM
     movf  BufOutEEP,0
     movwf LEDS
     movf  Temp1,0
     addlw .1
     movwf AdresEEP
     call  ReadEEPROM
     movf  BufOutEEP,0
     movwf ELED1
     movf  Temp1,0
     addlw .2
     movwf AdresEEP
     call  ReadEEPROM
     movf  BufOutEEP,0
     movwf ELED2
     movf  Temp1,0
     addlw .3
     movwf AdresEEP
     call  ReadEEPROM
     movf  BufOutEEP,0
     movwf ELED3
     movf Temp1,0
     addlw .4
     movwf AdresEEP
     call  ReadEEPROM
     movf  BufOutEEP,0
     movwf ELED4
     movf  Temp1,0
     addlw .5
     movwf AdresEEP
     call  ReadEEPROM
     movf  BufOutEEP,0
     movwf ELED5
     movf  Temp1,0
     addlw .6
     movwf AdresEEP
     call  ReadEEPROM
     movf  BufOutEEP,0
     movwf ELEDKM
     return
;----------------------------------------------------------------------;
;                   Write byte ELES..ELED5                              ;
;----------------------------------------------------------------------;
WriteByte
     clrf  Temp1
     movf  Marker,0
     movwf Temp2
LoopW
     movlw  .7
     addwf  Temp1,1
     decfsz Temp2,1
     goto   LoopW
     movf  Temp1,0
     movwf AdresEEP
     movf  LEDS,0
     movwf BufInEEP
     call  WriteEEPROM
     movf  Temp1,0
     addlw .1
     movwf AdresEEP
     movf  ELED1,0
     movwf BufInEEP
     call  WriteEEPROM
     movf  Temp1,0
     addlw .2
     movwf AdresEEP
     movf  ELED2,0
     movwf BufInEEP
     call  WriteEEPROM
     movf  Temp1,0
     addlw .3
     movwf AdresEEP
     movf  ELED3,0
     movwf BufInEEP
     call  WriteEEPROM
     movf  Temp1,0
     addlw .4
     movwf AdresEEP
     movf  ELED4,0
     movwf BufInEEP
     call  WriteEEPROM
     movf  Temp1,0
     addlw .5
     movwf AdresEEP
     movf  ELED5,0
     movwf BufInEEP
     call  WriteEEPROM
     movf  Temp1,0
     addlw .6
     movwf AdresEEP
     movf  ELEDKM,0
     movwf BufInEEP
     call  WriteEEPROM
     return
;----------------------------------------------------------------------;
;                              Control menu                            ;
;----------------------------------------------------------------------;
Control
     movlw  .1
     xorwf  Select,0
     btfsc  STATUS,Z
     call   Menu1
     movlw  .2
     xorwf  Select,0
     btfsc  STATUS,Z
     call   Menu2
     movlw  .3
     xorwf  Select,0
     btfsc  STATUS,Z
     call   Menu3
     call  ScanKP
     movlw D'100'
     call   nmsec
     
     movlw  .1
     xorwf  Select,0
     btfss  STATUS,Z
     call   ScanK1
     movlw  .1
     xorwf  Select,0
     btfsc  STATUS,Z
     call   ScanKey
     goto   Restart
;----------------------------------------------------------------------;
;                              Control menu1                           ;
;----------------------------------------------------------------------;
Menu1
     movlw .0
     xorwf Marker,0
     btfsc STATUS,Z
     bsf   Marker,0
     call  ReadByte
     bcf   LEDS,7
     call  OutMarkDis
     call  Line1
     call  Text5
     call  SymbMark
     movlw .0
     xorwf ELEDKM,0
     btfsc STATUS,Z
     call  FreqkHz
     movlw .1
     xorwf ELEDKM,0
     btfsc STATUS,Z
     call  FreqMHz1
     movlw .2
     xorwf ELEDKM,0
     btfsc STATUS,Z
     call  FreqMHz2
     call  Line2
     call  Text2
     return   
;----------------------------------------------------------------------;
;                              Control menu2                           ;
;----------------------------------------------------------------------;
Menu2
       
     btfsc Key2
     call  Zummer
     btfsc Key3
     call  Zummer
     
     btfsc Key3
     incf  SelNum,1
     
     movlw .7
     xorwf SelNum,0
     btfsc STATUS,Z
     clrf  SelNum
     btfss TempR,0
     call  ReadByte
     
     call  OutMarkDis
     
     
     call  BLeds
     
     call  Line1
     call  Text5
     
     call  SymbMark
     movlw .0
     xorwf ELEDKM,0
     btfsc STATUS,Z
     call  FreqkHz
     movlw .1
     xorwf ELEDKM,0
     btfsc STATUS,Z
     call  FreqMHz1
     movlw .2
     xorwf ELEDKM,0
     btfsc STATUS,Z
     call  FreqMHz2
     call  Line2
     call  Text4
     btfsc Key3
     
     call  ScanKP
     bsf   TempR,0
     return   
;----------------------------------------------------------------------;
;                              Control menu3                           ;
;----------------------------------------------------------------------;
Menu3
     btfsc  Key2
     call   Zummer
     btfsc  Key3
     call   Zummer
       
     call  Line1
     call  Text6
     call  Line2
     call  Text7
     btfsc Key3
     clrf  Select
     btfsc Key3
     clrf  Marker
     btfsc Key3
     goto  Restart
     btfsc Key2
     call  WriteByte
     btfsc Key2
     clrf  Marker
     btfsc Key2
     clrf  Select
     btfsc Key2
     goto  Restart
     return 
;----------------------------------------------------------------------;
;                                Beep Leds                             ;
;----------------------------------------------------------------------;
BLeds
     btfsc Key2
     return
     bcf  LEDS,7
     incf Time1,1
     btfsc Time1,3
     return
     movlw .0
     xorwf SelNum,0
     btfsc STATUS,Z
     bsf   LEDS,7
     movlw .1
     xorwf SelNum,0
     btfsc STATUS,Z
     movlw .239
     btfsc STATUS,Z
     movwf LED1
     movlw .2
     xorwf SelNum,0
     btfsc STATUS,Z
     movlw .239
     btfsc STATUS,Z
     movwf LED2
     movlw .3
     xorwf SelNum,0
     btfsc STATUS,Z
     movlw  .239
     btfsc STATUS,Z
     movwf LED3
     movlw .4
     xorwf SelNum,0
     btfsc STATUS,Z
     movlw .239
     btfsc STATUS,Z
     movwf LED4
     movlw .5
     xorwf SelNum,0
     btfsc STATUS,Z
     movlw .239
     btfsc STATUS,Z
     movwf LED5
     return
;----------------------------------------------------------------------;
;                   Conver 5 leds to 7 leds (EEPROM markers)           ;
;----------------------------------------------------------------------;
ConvEEP
     call  ReadByte
     movlw .0
     xorwf ELEDKM,0
     btfsc STATUS,Z
     call  kHz5_7
     movlw .1
     xorwf ELEDKM,0
     btfsc STATUS,Z
     call  MHz1_5_7
     movlw .2
     xorwf ELEDKM,0
     btfsc STATUS,Z
     call  MHz2_5_7
     call  ConvFr
     btfss LEDS,0
     call  PlusD
     btfsc LEDS,0
     call  MinusD
     call  DisRes
     return
kHz5_7
     movlw 0xA1
     movwf FSR
     movf  ELED1,0
     movwf INDF
     movlw 0xA2
     movwf FSR
     movf  ELED2,0
     movwf INDF
     movlw 0xA3
     movwf FSR
     movf  ELED3,0
     movwf INDF
     movlw 0xA4
     movwf FSR
     movf  ELED4,0
     movwf INDF
     movlw 0xA5
     movwf FSR
     movf  ELED5,0
     movwf INDF
     movlw 0x9F
     movwf FSR
     clrf INDF
     movlw 0xA0
     movwf FSR
     clrf INDF
     return
MHz1_5_7
     movlw 0xA0
     movwf FSR
     movf  ELED1,0
     movwf INDF
     movlw 0xA1
     movwf FSR
     movf  ELED2,0
     movwf INDF
     movlw 0xA2
     movwf FSR
     movf  ELED3,0
     movwf INDF
     movlw 0xA3
     movwf FSR
     movf  ELED4,0
     movwf INDF
     movlw 0xA4
     movwf FSR
     movf  ELED5,0
     movwf INDF
     movlw 0xA5
     movwf FSR
     clrf INDF
     movlw 0x9F
     movwf FSR
     clrf INDF
     return
MHz2_5_7
     movlw 0x9F
     movwf FSR
     movf  ELED1,0
     movwf INDF
     movlw 0xA0
     movwf FSR
     movf  ELED2,0
     movwf INDF
     movlw 0xA1
     movwf FSR
     movf  ELED3,0
     movwf INDF
     movlw 0xA2
     movwf FSR
     movf  ELED4,0
     movwf INDF
     movlw 0xA3
     movwf FSR
     movf  ELED5,0
     movwf INDF
     movlw 0xA4
     movwf FSR
     clrf INDF
     movlw 0xA5
     movwf FSR
     clrf INDF
     return
;----------------------------------------------------------------------;
;                         Conver 5 leds to 7 leds (Freq)               ;
;----------------------------------------------------------------------;
ConvFr
     movlw .0
     xorwf SelFr,0
     btfsc STATUS,Z
     call  kHz5_7f
     movlw .1
     xorwf SelFr,0
     btfsc STATUS,Z
     call  MHz1_5_7f
     movlw .2
     xorwf SelFr,0
     btfsc STATUS,Z
     call  MHz2_5_7f
     return
kHz5_7f
     movlw 0xA9
     movwf FSR
     movf  TenK,0
     movwf INDF
     movlw 0xAA
     movwf FSR
     movf  Thou,0
     movwf INDF
     movlw 0xAB
     movwf FSR
     movf  Hund,0
     movwf INDF
     movlw 0xAC
     movwf FSR
     movf  Tens,0
     movwf INDF
     movlw 0xAD
     movwf FSR
     movf  Ones,0
     movwf INDF
     movlw 0xA7
     movwf FSR
     clrf INDF
     movlw 0xA8
     movwf FSR
     clrf INDF
     return
MHz1_5_7f
     movlw 0xA8
     movwf FSR
     movf  TenK,0
     movwf INDF
     movlw 0xA9
     movwf FSR
     movf  Thou,0
     movwf INDF
     movlw 0xAA
     movwf FSR
     movf  Hund,0
     movwf INDF
     movlw 0xAB
     movwf FSR
     movf  Tens,0
     movwf INDF
     movlw 0xAC
     movwf FSR
     movf  Ones,0
     movwf INDF
     movlw 0xA7
     movwf FSR
     clrf INDF
     movlw 0xAD
     movwf FSR
     clrf INDF
     return
MHz2_5_7f
     movlw 0xA7
     movwf FSR
     movf  TenK,0
     movwf INDF
     movlw 0xA8
     movwf FSR
     movf  Thou,0
     movwf INDF
     movlw 0xA9
     movwf FSR
     movf  Hund,0
     movwf INDF
     movlw 0xAA
     movwf FSR
     movf  Tens,0
     movwf INDF
     movlw 0xAB
     movwf FSR
     movf  Ones,0
     movwf INDF
     movlw 0xAC
     movwf FSR
     clrf INDF
     movlw 0xAD
     movwf FSR
     clrf INDF
     return
;----------------------------------------------------------------------;
;                        Plus EEP data and Frequ                       ;
;----------------------------------------------------------------------;
PlusD
     movlw  .7
     movwf  Temp1
LoopPC
     decf   Temp1,1
     movf   Temp1,0
     addlw  0xAE
     movwf  FSR
     clrf   INDF
     movlw  .0
     xorwf  Temp1,0
     btfss  STATUS,Z
     goto   LoopPC
     movlw  .7
     movwf  Temp1
LoopPl
     decf   Temp1,1
     movf   Temp1,0
     addlw  0x9F
     movwf  FSR
     movf   INDF,0
     movwf  Temp2
     movf   Temp1,0
     addlw  0xA7
     movwf  FSR
     movf   INDF,0
     addwf  Temp2,0
     movwf  Temp3
     sublw  .9
     btfss  STATUS,C
     call   Plusf
     movf   Temp1,0
     addlw  0xAE
     movwf  FSR
     movf   Temp3,0
     addwf  INDF,1
     movlw  .0
     xorwf  Temp1,0
     btfss  STATUS,Z
     goto   LoopPl
     return
Plusf
     movlw  .10
     subwf  Temp3,1
     movf   Temp1,0
     addlw  0x9E
     movwf  FSR
     movlw  .1
     addwf  INDF,1
     return
;----------------------------------------------------------------------;
;                        Minus EEP data in Frequ                       ;
;----------------------------------------------------------------------;
MinusD
     movlw  .7
     movwf  Temp1
     clrf   Temp4
LoopMC
     decf   Temp1,1
     movf   Temp1,0
     addlw  0xAE
     movwf  FSR
     clrf   INDF
     movlw  .0
     xorwf  Temp1,0
     btfss  STATUS,Z
     goto   LoopMC
     movlw 0xA6
     movwf  FSR
     clrf   INDF
     
     movlw  .7
     movwf  Temp1
LoopMl
     decf   Temp1,1
     movf   Temp1,0
     addlw  0xA7
     movwf  FSR
     movf   INDF,0
     movwf  Temp2
     movf   Temp1,0
     addlw  0x9F
     movwf  FSR
     movf   INDF,0
     movwf  Temp3
     subwf  Temp2,0
     btfss  STATUS,C
     call   Minusf
     movf   Temp1,0
     addlw  0xA7
     movwf  FSR
     movf   INDF,0
     movwf  Temp2
     movf   Temp1,0
     addlw  0xAE
     movwf  FSR
     movf   Temp3,0
     subwf  Temp2,0
     movwf  INDF
     movlw  .0
     xorwf  Temp1,0
     btfss  STATUS,Z
     goto   LoopMl
     
     return
Minusf
     movf   Temp1,0
     addlw  0xA7
     movwf  FSR
     movlw  .10
     addwf  INDF,1
     movf   Temp1,0
    
     movwf  Temp4
Loopmg1
     decf   Temp4,1
     movlw  .255
     xorwf  Temp4,0
     btfsc  STATUS,Z
     return
     
     movf   Temp4,0
     addlw  0XA7
     movwf  FSR
     movf   INDF,0
     movlw  .0
     xorwf  INDF,0
     btfsc  STATUS,Z
     goto   Loopmg2
     movlw  .1
     subwf  INDF,1
     return
Loopmg2
     
     movlw  .9
     movwf  INDF
     goto   Loopmg1  
;----------------------------------------------------------------------;
;                         Out to dispaly resultats                     ;
;----------------------------------------------------------------------;
DisRes
     movlw  0XAE
     movwf  FSR
     movlw  .9
     xorwf  INDF,0
     btfsc  STATUS,Z
     goto   ErrM
     movlw  0xAE
     movwf  FSR
     movlw  .0
     xorwf  INDF,0
     btfss  STATUS,Z
     goto   Res1
     movlw  0xAF
     movwf  FSR
     movlw  .0
     xorwf  INDF,0
     btfss  STATUS,Z
     goto   Res2
     movlw  0xB0
     movwf  FSR
     movlw  .0
     xorwf  INDF,0
     btfss  STATUS,Z
     goto   Res3
     goto   Res3
Res1 
     movlw  0xAE
     movwf  FSR
     movf   INDF,0
     movwf  LED1
     movlw  0xAF
     movwf  FSR
     movf   INDF,0
     movwf  LED2
     movlw  0xB0
     movwf  FSR
     movf   INDF,0
     movwf  LED3
     movlw  0xB1
     movwf  FSR
     movf   INDF,0
     movwf  LED4
     movlw  0xB2
     movwf  FSR
     movf   INDF,0
     movwf  LED5
     movlw  .2
     movwf  SelFr
     return
Res2
     movlw  0xAF
     movwf  FSR
     movf   INDF,0
     movwf  LED1
     movlw  0xB0
     movwf  FSR
     movf   INDF,0
     movwf  LED2
     movlw  0xB1
     movwf  FSR
     movf   INDF,0
     movwf  LED3
     movlw  0xB2
     movwf  FSR
     movf   INDF,0
     movwf  LED4
     movlw  0xB3
     movwf  FSR
     movf   INDF,0
     movwf  LED5
     movlw  .1
     movwf  SelFr
     return
Res3
     movlw  0xB0
     movwf  FSR
     movf   INDF,0
     movwf  LED1
     movlw  0xB1
     movwf  FSR
     movf   INDF,0
     movwf  LED2
     movlw  0xB2
     movwf  FSR
     movf   INDF,0
     movwf  LED3
     movlw  0xB3
     movwf  FSR
     movf   INDF,0
     movwf  LED4
     movlw  0xB4
     movwf  FSR
     movf   INDF,0
     movwf  LED5
     movlw  .0
     movwf   SelFr
     return
ErrM
     movlw  .253
     movwf  LED1
     movlw  .253
     movwf  LED2
     movlw  .253
     movwf  LED3
     movlw  .253
     movwf  LED4
     movlw  .253
     movwf  LED5
     movlw  .0
     movwf   SelFr
     return
;----------------------------------------------------------------------;
;                           The Main routine                           ;
;----------------------------------------------------------------------;


main:
       call ClearVar
       call init               ; initialize ports, set up timer
       call initlcd            ; initialize the LCD
       call Zummer
Restart
        movlw  .4
        xorwf  Select,0
        btfsc  STATUS,Z
        movlw  .3
        btfsc  STATUS,Z
        movwf  Select
        movlw  .255
        xorwf  Select,0
        btfsc  STATUS,Z
        clrf   Select
        movlw  .10
        xorwf  Marker,0
        btfsc  STATUS,Z
        movlw  .9
        btfsc  STATUS,Z
        movwf  Marker
        movlw  .255
        xorwf  Marker,0
        btfsc  STATUS,Z
        clrf   Marker
        movlw  .0
        xorwf  Select,0
        btfss  STATUS,Z
        goto   Control       
       call  Line1
       movlw .0
       xorwf Marker,0
       btfss STATUS,Z
       call  Text3
       movlw .0
       xorwf Marker,0
       btfsc STATUS,Z
       call  Text1               
       
       call  Count_kHz
       btfsc PIR1,0
       call  Count_MHz1
       btfsc PIR1,0
       call  Count_MHz2
       
       call  OutFrDis
       movlw .0
       xorwf Marker,0
       btfss STATUS,Z
       call  ConvEEP
       movlw .0
       xorwf SelFr,0
       btfsc STATUS,Z
       call  FreqkHz
       btfsc SelFr,0
       call  FreqMHz1
       btfsc SelFr,1
       call  FreqMHz2
    
       call  Line2               
       call  Text2
       call  ScanKP
       movlw D'100'
       call   nmsec

       call  ScanKey
       
       clrf  SelNum
       bcf   TempR,0 
       goto  Restart

CLV        ORG      0x0800
           clrf     Dlay            ; 8 Bit Delay Variable
           clrf     Temp            ; a temporary variable
           clrf     cntmsec
           clrf     LED1
           clrf     LED2
           clrf     LED3
           clrf     LED4
           clrf     LED5
           clrf     ELED1
           clrf     ELED2
           clrf     ELED3
           clrf     ELED4
           clrf     ELED5
           clrf     NumH
           clrf     NumL
           clrf     Tens
           clrf     Ones
           clrf     Thou
           clrf     TenK
           clrf     Hund
           clrf     FrHig
           clrf     FrLow
           clrf     Temp1
           clrf     Temp2
           clrf     Temp3
           clrf     Temp4
           clrf     SelFr
           clrf     Marker
           clrf     Select
           clrf     AdresEEP
           clrf     BufOutEEP
           clrf     BufInEEP
           clrf     SelNum
           clrf     ELEDKM
           clrf     Time1
           clrf     TempR
           clrf     LEDS
           bcf      PCLATH,3
           return
ScanKeyR   ORG    0x0900
           btfsc    Key1
           goto     SERL    
           btfsc    Key2
           goto     SERL 
           btfsc    Key3
           goto     SERL
           bcf     PCLATH,3
           bcf     PCLATH,0
           return
SERL
           bcf     PCLATH,3
           bcf     PCLATH,0
           goto    SER          
       end 


